@@ -43,6 +43,7 @@ import apt
 import aptsources.sourceslist
 import apt_pkg
 import time
+import fcntl
 
 from apt_daemon import AptDaemon,AppActions
 from apt_daemon import WorkitemError
@@ -60,7 +61,6 @@ DEB_SOURCE_UBUNTUKYLIN = "deb " + HTTP_SOURCE_UBUNTUKYLIN
 UBUNTUKYLIN_SOFTWARECENTER_ACTION = 'com.ubuntukylin.softwarecenter.action'
 
 
-
 class WorkItem:
      def __init__(self, pkgname, action, kwargs):
         self.pkgname = pkgname
@@ -73,18 +73,28 @@ class WorkThread(threading.Thread):
     def __init__(self, dbus_service):
         threading.Thread.__init__(self)
         self.dbus_service = dbus_service
+        self.thread_is_working = 0
+        self.uksc_is_working = 0
 
     def run(self):
 #        print "The backend start the work thread..."
         while(True):
-
+            if self.thread_is_working <100:
+                self.thread_is_working += 1
+            else:
+                self.thread_is_working = 0    
             if len(self.dbus_service.worklist) == 0:
                 time.sleep(0.5)
                 continue
 
+            if is_file_locked("/var/lib/dpkg/lock") is True:
+                time.sleep(0.5)
+                continue
+
             self.dbus_service.mutex.acquire()
             item = self.dbus_service.worklist.pop(0) # pop(0) is get first item and remove it from list
             self.dbus_service.mutex.release()
+            self.uksc_is_working = 1
 
             try:
                 func = getattr(self.dbus_service.daemonApt,item.action)
@@ -96,14 +106,51 @@ class WorkThread(threading.Thread):
                     print "Action exec failed..."
             except WorkitemError as e:
                 # print(e.errornum)
+                self.uksc_is_working = 0
                 kwarg = {"apt_appname": item.pkgname,
                         "apt_percent": str(-e.errornum),
                         "action": str(item.action),
                         }
                 self.dbus_service.software_apt_signal("apt_error", kwarg)
+            except:
+                self.uksc_is_working = 0
+                kwarg = {"apt_appname": item.pkgname,
+                        "apt_percent": str(-6.6),
+                        "action": str(item.action),
+                        }
+                self.dbus_service.software_apt_signal("apt_error", kwarg)
 
 #            print "finish one acion....."
-            time.sleep(0.5)
+            #time.sleep(0.5)
+
+
+def is_file_locked(lockfile):
+    """
+    Check whether ``apt-get`` or ``dpkg`` is currently active by check the lock file.
+
+    This works by checking whether the lock file like ``/var/lib/dpkg/lock``
+    ``/var/lib/apt/lists/lock`` is locked by an ``apt-get`` or ``dpkg`` process,
+    which in turn is done by momentarily trying to acquire the lock.
+     This means that the current process needs to have sufficient privileges.
+
+    :returns: ``True`` when the lock is already taken (``apt-get`` or ``dpkg``
+              is running), ``False`` otherwise.
+    :raises: :py:exc:`exceptions.IOError` if the required privileges are not
+             available.
+
+    .. note:: ``apt-get`` doesn't acquire this lock until it needs it, for
+              example an ``apt-get update`` run consists of two phases (first
+              fetching updated package lists and then updating the local
+              package index) and only the second phase claims the lock (because
+              the second phase writes the local package index which is also
+              read from and written to by ``dpkg``).
+    """
+    with open(lockfile, 'w') as handle:
+        try:
+            fcntl.lockf(handle, fcntl.LOCK_EX | fcntl.LOCK_NB)
+            return False
+        except IOError:
+            return True
 
 class SoftwarecenterDbusService(dbus.service.Object):
 
@@ -154,34 +201,38 @@ class SoftwarecenterDbusService(dbus.service.Object):
         self.mutex.release()
         print "####add_worker_item finished!"
 
-    def del_worker_item_by_name(self, pkgname):
-        print "####del_worker_item_by_name:",pkgname
-        exist = False
+    def del_worker_item_by_name(self, cancelinfo):
+        print "####del_worker_item_by_name:",cancelinfo[0]
+
+        del_work_item = None
         self.mutex.acquire()
-        for item in self.worklist:
-            if item.pkgname == pkgname:
-                exist = True
-                break
-        if exist is True:
-            self.worklist.remove(pkgname)
+        try:
+            for item in self.worklist:
+                if item.pkgname == cancelinfo[0] and item.action == cancelinfo[1]:
+                    self.worklist.remove(item)
+                    del_work_item = item
+                    break
+        except:
+            pass
         self.mutex.release()
 
         self.cancelmutex.acquire()
-        self.cancel_name_list.append(pkgname)
+        if del_work_item != None:
+            self.cancel_name_list.append(del_work_item)
         self.cancelmutex.release()
         print "####del_worker_item_by_name finished!"
 
-    def check_cancel_worker_item(self, pkgname):
+    def check_cancel_worker_item(self, cancelinfo):
 #        print "####check_cancel_worker_item:",pkgname
         cancel = False
         self.cancelmutex.acquire()
 #        print "check_cancel_worker_item:",len(self.cancel_name_list)
+
         for item in self.cancel_name_list:
-            if item == pkgname:
+            if item.pkgname == cancelinfo[0] and item.action == cancelinfo[1]:
+                self.cancel_name_list.remove(item)
                 cancel = True
                 break
-        if cancel is True:
-            self.cancel_name_list.remove(pkgname)
         self.cancelmutex.release()
 
 #        print "####check_cancel_worker_item finished!:",cancel
@@ -349,27 +400,32 @@ class SoftwarecenterDbusService(dbus.service.Object):
         print "####upgrade return"
         return True
 
-    @dbus.service.method(INTERFACE, in_signature='s', out_signature='b', sender_keyword='sender')
-    def cancel(self, pkgName, sender=None):
-        print "####cancel: ",pkgName
+    @dbus.service.method(INTERFACE, in_signature='as', out_signature='s', sender_keyword='sender')
+    def cancel(self, cancelinfo, sender=None):
+        print "####cancel: ",cancelinfo[0]
 
         granted = self.auth_with_policykit(sender,UBUNTUKYLIN_SOFTWARECENTER_ACTION)
         if not granted:
-            return False
+            return "False"
 
-        self.del_worker_item_by_name(pkgName)
-
-        print "####cancel return"
-        return True
+        self.del_worker_item_by_name(cancelinfo)
+        if self.check_cancel_worker_item(cancelinfo) is True:
+            print "####cancel return"
+            return "True"
+        else:
+            return "False"
 
     # apt-get update sa:software_fetch_signal()
-    @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
+    @dbus.service.method(INTERFACE, in_signature='b', out_signature='s', sender_keyword='sender')
     def update(self, quiet, sender=None):
         print "####update: "
 
         granted = self.auth_with_policykit(sender,UBUNTUKYLIN_SOFTWARECENTER_ACTION,"要更新软件源")
         if not granted:
-            return False
+            return "False"
+
+        if is_file_locked("/var/lib/apt/lists/lock") is True:
+            return "Locked"
 
         kwargs = {"quiet":str(quiet),
                   }
@@ -379,8 +435,8 @@ class SoftwarecenterDbusService(dbus.service.Object):
         self.add_worker_item(item)
 
 #        self.daemonApt.update()
-
-        print "####update return"
+        #print "####update return"
+        return "True"
     #????????????????????????????
     # apt-get update sa:software_fetch_signal()
     @dbus.service.method(INTERFACE, in_signature='b', out_signature='b', sender_keyword='sender')
@@ -527,6 +583,41 @@ class SoftwarecenterDbusService(dbus.service.Object):
     def software_signal_test(self, msg):
         pass
 
+    @dbus.service.method(INTERFACE, in_signature='', out_signature='ai')
+    def check_work_item(self):
+        dpkg_is_running = 0
+        workitemcount = 0
+        self.mutex.acquire()
+        if len(self.worklist) != 0:
+            workitemcount = len(self.worklist)
+        self.mutex.release()
+        dpkg_is_running = is_file_locked("/var/lib/dpkg/lock")
+        if dpkg_is_running is True:
+            dpkg_is_running = 1
+        else:
+            dpkg_is_running = 0
+        res = [workitemcount, dpkg_is_running]
+        return res
+
+    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
+    def clear_all_work_item(self):
+        self.worklist = []
+        self.cancel_name_list = []
+        self.worker_thread.uksc_is_working = 0
+        
+    @dbus.service.method(INTERFACE, in_signature='', out_signature='i')
+    def check_dbus_thread_is_working(self):
+        return self.worker_thread.thread_is_working
+
+    @dbus.service.method(INTERFACE, in_signature='', out_signature='i')
+    def check_uksc_is_working(self):
+        return self.worker_thread.uksc_is_working
+
+    @dbus.service.method(INTERFACE, in_signature='', out_signature='')
+    def set_uksc_not_working(self):
+        self.worker_thread.uksc_is_working = 0
+    
+
 if __name__ == '__main__':
     os.environ["TERM"] = "xterm"
     os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/X11R6/bin"
@@ -538,4 +629,4 @@ if __name__ == '__main__':
     mainloop = GObject.MainLoop()
     signal.signal(signal.SIGINT, lambda : mainloop.quit())
     SoftwarecenterDbusService(dbus.SystemBus(), mainloop)
-    mainloop.run()
+    mainloop.run()
\ No newline at end of file
