@@ -390,11 +390,12 @@ class AppManager(QObject):
     def get_advertisements(self):
         print "we need to get the advertisements"
         tmpads = []
-        tmpads.append(Advertisement("pchomewallpaper", "url", "ad0.png", "http://download.pchome.net/wallpaper/"))
-        tmpads.append(Advertisement("qq", "url", "ad1.png", "http://www.ubuntukylin.com/ukylin/forum.php?mod=viewthread&tid=7688&extra=page%3D1"))
-        tmpads.append(Advertisement("wps", "pkg", "ad2.png", "wps-office"))
-        tmpads.append(Advertisement("dota2", "url", "ad3.png", "http://www.ubuntukylin.com/ukylin/forum.php?mod=viewthread&tid=7687&extra=page%3D1"))
-        tmpads.append(Advertisement("pps", "url", "ad4.png", "http://dl.pps.tv/pps_linux_download.html"))
+        tmpads.append(Advertisement("pchomewallpaper", "url", "ad0.png", "adbground0.png", "http://download.pchome.net/wallpaper/"))
+        tmpads.append(Advertisement("qq", "url", "ad1.png", "adbground1.png", "http://www.ubuntukylin.com/ukylin/forum.php?mod=viewthread&tid=7688&extra=page%3D1"))
+        tmpads.append(Advertisement("wps", "pkg", "ad2.png", "adbground2.png", "wps-office"))
+        tmpads.append(Advertisement("dota2", "url", "ad3.png", "adbground3.png", "http://www.ubuntukylin.com/ukylin/forum.php?mod=viewthread&tid=7687&extra=page%3D1"))
+        tmpads.append(Advertisement("pps", "url", "ad4.png", "adbground4.png", "http://dl.pps.tv/pps_linux_download.html"))
+        tmpads.append(Advertisement("pchomewallpaper", "url", "ad5.png", "adbground5.png", "http://download.pchome.net/wallpaper/"))
         self.emit(Signals.ads_ready,tmpads)
 
     #get apps in ubuntukylin archives, this is now implemented with config file
@@ -424,7 +425,8 @@ class AppManager(QObject):
         item = WorkerItem("get_reviews",kwargs)
 
         app = self.get_application_by_name(pkgname)
-        reviews = app.get_reviews(page)
+        if app is not None:
+            reviews = app.get_reviews(page)
 
         # force == True means need get review from server immediately
         if reviews is not None and force == False:
@@ -472,14 +474,20 @@ class AppManager(QObject):
             page = item.kwargs['page']
 
             app = self.get_application_by_name(item.kwargs['packagename'])
-            app.add_reviews(page,reviews)
+            if app is not None:
+                app.add_reviews(page,reviews)
+            else:
+                print item.kwargs['packagename'], " not exist"
 
             self.emit(Signals.app_reviews_ready,reviews)
         elif item.funcname == "get_screenshots":
             LOG.debug("screenshots ready:%s",len(reslist))
             screenshots = reslist
             app = self.get_application_by_name(item.kwargs['packagename'])
-            app.screenshots = screenshots
+            if app is not None:
+                app.screenshots = screenshots
+            else:
+                print item.kwargs['packagename'], " not exist"
 
             self.emit(Signals.app_screenshots_ready,screenshots)
         elif item.funcname == "get_rating_review_stats":
@@ -624,7 +632,10 @@ class AppManager(QObject):
             app = self.get_application_by_name(rr[0])
             if(app is not None):
                 app.ratingrank = rr[1]
-                applist.append(app)
+                try:
+                    applist.index(app)
+                except:
+                    applist.append(app)
 
         self.emit(Signals.ratingrank_ready,applist)
 
