@@ -95,6 +95,7 @@ class NormalCard(QWidget):
 
         iconpath = commontools.get_icon_path(self.app.name)
         self.ui.icon.setStyleSheet("QLabel{background-image:url('" + iconpath + "')}")
+        self.ui.progressBar_icon.setStyleSheet("QLabel{background-image:url('" + iconpath + "')}")
 
         # self.ui.baseWidget.setStyleSheet("QWidget{border:0px;}")
         self.ui.name.setStyleSheet("QLabel{font-size:13px;font-weight:bold;color:#666666;}")
@@ -146,36 +147,62 @@ class NormalCard(QWidget):
         self.star.move(75, 56)
 
         # btn & border
-        if self.app.status == PkgStates.INSTALLING:
-            self.ui.btn.setEnabled(False)
-            self.ui.btn.setText("正在安装")
-            self.star.show()
-            self.ui.isInstalled.setVisible(False)
-            self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-install-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-install-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-install-btn-3.png');}")
-            self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-install-border.png');}")
-        elif self.app.status == PkgStates.REMOVING:
-            self.ui.btn.setEnabled(False)
-            self.ui.btn.setText("正在卸载")
-            if (Globals.NOWPAGE in (PageStates.UPPAGE,PageStates.UNPAGE,PageStates.SEARCHUPPAGE,PageStates.SEARCHUNPAGE)):
+        if self.app.percent > 0 or (self.app.status in (PkgStates.INSTALLING, PkgStates.REMOVING, PkgStates.UPGRADING) and self.app.percent == 0):
+            self.ui.progressBar.setVisible(True)
+            self.ui.progresslabel.setVisible(True)
+            self.ui.progressBar_icon.setVisible(True)
+            self.ui.progressBar.setValue(self.app.percent)
+            self.ui.progresslabel.setText(str('%.0f' % self.app.percent) + '%')
+
+            if self.app.status == PkgStates.INSTALLING:
+                self.ui.btn.setEnabled(False)
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在安装")
+                else:
+                    self.ui.btn.setText("等待安装")
                 self.star.show()
+
                 self.ui.isInstalled.setVisible(False)
-            else:
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                             "QProgressBar:chunk{background-color:#BBF9A3;}")
+                self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-install-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-install-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-install-btn-3.png');}")
+                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-install-border.png');}")
+            elif self.app.status == PkgStates.REMOVING:
+                self.ui.btn.setEnabled(False)
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在卸载")
+                else:
+                    self.ui.btn.setText("等待卸载")
+                if (Globals.NOWPAGE in (PageStates.UPPAGE,PageStates.UNPAGE,PageStates.SEARCHUPPAGE,PageStates.SEARCHUNPAGE)):
+                    self.star.show()
+                    self.ui.isInstalled.setVisible(False)
+                else:
+                    self.star.hide()
+                    self.ui.isInstalled.setVisible(True)
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                                  "QProgressBar:chunk{background-color:#C5CED9;}")
+                self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
+                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
+            elif self.app.status == PkgStates.UPGRADING:
+                self.ui.btn.setEnabled(False)
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在升级")
+                else:
+                    self.ui.btn.setText("等待升级")
                 self.star.hide()
-                self.ui.isInstalled.setVisible(True)
-            self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
-            self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
-        elif self.app.status == PkgStates.UPGRADING:
-            self.ui.btn.setEnabled(False)
-            self.ui.btn.setText("正在升级")
+                if (Globals.NOWPAGE in (PageStates.UPPAGE,PageStates.UNPAGE,PageStates.SEARCHUPPAGE,PageStates.SEARCHUNPAGE)):
+                    self.star.show()
+                    self.ui.isInstalled.setVisible(False)
+                else:
+                    self.star.hide()
+                    self.ui.isInstalled.setVisible(True)
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                             "QProgressBar:chunk{background-color:#FDD99A;}")
+                self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
+                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
+
             self.star.hide()
-            if (Globals.NOWPAGE in (PageStates.UPPAGE,PageStates.UNPAGE,PageStates.SEARCHUPPAGE,PageStates.SEARCHUNPAGE)):
-                self.star.show()
-                self.ui.isInstalled.setVisible(False)
-            else:
-                self.star.hide()
-                self.ui.isInstalled.setVisible(True)
-            self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
-            self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
+
         else:
             if(Globals.NOWPAGE == PageStates.ALLPAGE or Globals.NOWPAGE == PageStates.HOMEPAGE ):
                 if(self.app.is_installed):
@@ -281,6 +308,16 @@ class NormalCard(QWidget):
                     self.ui.btn.setText("无法卸载")
                     self.ui.btn.setEnabled(False)
 
+            if self.app.percent < 0:
+                self.star.hide()
+                self.ui.progressBar.setVisible(True)
+                self.ui.progresslabel.setVisible(True)
+                self.ui.progressBar_icon.setVisible(True)
+                self.ui.progressBar.setValue(0)
+                self.ui.progresslabel.setText("失败")
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                             "QProgressBar:chunk{background-color:#BBF9A3;}")
+
         self.ui.btn.clicked.connect(self.slot_btn_click)
         self.ui.btnDetail.clicked.connect(self.slot_emit_detail)
 
@@ -345,7 +382,8 @@ class NormalCard(QWidget):
             self.ui.btn.setEnabled(False)
             if(self.ui.btn.text() == "安装"):
                 self.app.status = PkgStates.INSTALLING
-                self.ui.btn.setText("正在安装")
+                self.ui.btn.setText("等待安装")
+                self.slot_show_progress("install")
                 self.emit(Signals.install_app, self.app)
                 self.emit(Signals.get_card_status, self.app.name, PkgStates.INSTALLING)
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-install-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-install-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-install-btn-3.png');}")
@@ -353,19 +391,99 @@ class NormalCard(QWidget):
 
             elif(self.ui.btn.text() == "升级"):
                 self.app.status = PkgStates.UPGRADING
-                self.ui.btn.setText("正在升级")
+                self.ui.btn.setText("等待升级")
+                self.slot_show_progress("upgrade")
                 self.emit(Signals.upgrade_app, self.app)
                 self.emit(Signals.get_card_status, self.app.name, PkgStates.UPGRADING)
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
 
             elif(self.ui.btn.text() == "卸载"):
-                self.app.status = PkgStates.REMOVING
+                if self.app.name == "ubuntu-kylin-software-center":
+                    self.emit(Signals.uninstall_uksc_or_not, "normalcard")
+                else:
+                    self.app.status = PkgStates.REMOVING
+                    self.ui.btn.setText("等待卸载")
+                    self.slot_show_progress("remove")
+                    self.emit(Signals.remove_app, self.app)
+                    self.emit(Signals.get_card_status, self.app.name, PkgStates.REMOVING)
+                    self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
+                    self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
+
+    def uninstall_uksc(self, where):
+        if where == "normalcard":
+            self.app.status = PkgStates.REMOVING
+            self.ui.btn.setText("等待卸载")
+            self.emit(Signals.remove_app, self.app)
+            self.emit(Signals.get_card_status, self.app.name, PkgStates.REMOVING)
+            self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
+            self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
+
+    def cancel_uninstall_uksc(self, where):
+        if where == "normalcard":
+            self.ui.btn.setEnabled(True)
+
+    # wb
+    def slot_show_progress(self,status):
+        self.ui.progressBar.setVisible(True)
+        self.ui.progresslabel.setVisible(True)
+        self.ui.progressBar_icon.setVisible(True)
+        self.star.setVisible(False)
+        if status == "install":
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#BBF9A3;}")
+        elif status == "upgrade":
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#FDD99A;}")
+        else :
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                            "QProgressBar:chunk{background-color:#C5CED9;}")
+        #self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+        #                                    "QProgressBar:chunk{background-color:#5DC4FE;}")#text-align:right;
+        self.ui.progressBar.setRange(0,100)
+        self.ui.progresslabel.setText(str(0) + '%')
+        self.ui.progressBar.reset()
+
+    def slot_progress_change(self, pkgname, percent, status):
+        if self.app.name == pkgname:
+            self.ui.progressBar.setVisible(True)
+            self.ui.progresslabel.setVisible(True)
+            self.ui.progressBar_icon.setVisible(True)
+            self.star.setVisible(False)
+            if status == AppActions.INSTALL:
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                             "QProgressBar:chunk{background-color:#BBF9A3;}")
+                self.ui.btn.setText("正在安装")
+            elif status == AppActions.UPGRADE:
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                             "QProgressBar:chunk{background-color:#FDD99A;}")
+                self.ui.btn.setText("正在升级")
+            elif status == AppActions.REMOVE:
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                           "QProgressBar:chunk{background-color:#C5CED9;}")
                 self.ui.btn.setText("正在卸载")
-                self.emit(Signals.remove_app, self.app)
-                self.emit(Signals.get_card_status, self.app.name, PkgStates.REMOVING)
-                self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
-                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
+            self.ui.progressBar.setValue(percent)
+            if percent < float(0.0):
+                self.ui.progressBar.setValue(0)
+                self.ui.progresslabel.setText("失败")
+            else:
+                self.ui.progresslabel.setText(str('%.0f' % percent) + '%')
+
+    def slot_progress_finish(self,pkgname):
+        if self.app.name == pkgname:
+            self.ui.progresslabel.setVisible(False)
+            self.ui.progressBar_icon.setVisible(False)
+            self.ui.progressBar.setVisible(False)
+            #self.star.setVisible(True)
+            self.ui.progressBar.reset()
+
+    def slot_progress_cancel(self, pkgname):
+        if self.app.name == pkgname:
+            self.ui.progresslabel.setVisible(False)
+            self.ui.progressBar_icon.setVisible(False)
+            self.ui.progressBar.setVisible(False)
+            #self.star.setVisible(True)
+            self.ui.progressBar.reset()
 
     # kobe 1106
     def slot_change_btn_status(self, pkgname, status):
@@ -373,18 +491,27 @@ class NormalCard(QWidget):
             self.ui.btn.setEnabled(False)
             if status == PkgStates.INSTALLING:
                 self.app.status = PkgStates.INSTALLING
-                self.ui.btn.setText("正在安装")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在安装")
+                else:
+                    self.ui.btn.setText("等待安装")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-install-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-install-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-install-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-install-border.png');}")
             elif status == PkgStates.REMOVING:
                 self.app.status = PkgStates.REMOVING
-                self.ui.btn.setText("正在卸载")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在卸载")
+                else:
+                    self.ui.btn.setText("等待卸载")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
 
             elif status == PkgStates.UPGRADING:
                 self.app.status = PkgStates.UPGRADING
-                self.ui.btn.setText("正在升级")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在升级")
+                else:
+                    self.ui.btn.setText("等待升级")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
 
@@ -458,7 +585,7 @@ class NormalCard(QWidget):
                     self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-run-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-run-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-run-btn-3.png');}")
                     self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-run-border.png');}")
 
-    def slot_work_cancel(self, pkgname,action):
+    def slot_work_cancel(self, pkgname, action):
         if self.app.name == pkgname:
             if action == AppActions.INSTALL:
                 self.app.status = PkgStates.INSTALL
@@ -494,3 +621,6 @@ class NormalCard(QWidget):
                 self.ui.btn.setEnabled(True)
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
+
+            if self.app.percent < 0:
+                self.star.hide()
