@@ -92,6 +92,7 @@ class RcmdCard(QWidget):
 
         iconpath = commontools.get_icon_path(self.app.name)
         self.ui.icon.setStyleSheet("QLabel{background-image:url('" + iconpath + "')}")
+        self.ui.progressBar_icon.setStyleSheet("QLabel{background-image:url('" + iconpath + "')}")
 
         # self.ui.baseWidget.setStyleSheet("QWidget{border:0px;}")
         self.ui.name.setStyleSheet("QLabel{font-size:13px;font-weight:bold;color:#666666;}")
@@ -240,28 +241,99 @@ class RcmdCard(QWidget):
             self.ui.btn.setEnabled(False)
             # self.ui.btn.setText("正在处理")
             self.app.status = PkgStates.INSTALLING
-            self.ui.btn.setText("正在安装")
+            self.ui.btn.setText("等待安装")
+            self.slot_show_progress("install")
             self.emit(Signals.install_app, self.app)
             self.emit(Signals.get_card_status, self.app.name, PkgStates.INSTALLING)
 
+    # wb
+    def slot_show_progress(self,status):
+        self.star.hide()
+        self.ui.progressBar.setVisible(True)
+        self.ui.progresslabel.setVisible(True)
+        self.ui.progressBar_icon.setVisible(True)
+        if status == "install":
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#BBF9A3;}")
+        elif status == "upgrade":
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#FDD99A;}")
+        else :
+            self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                            "QProgressBar:chunk{background-color:#C5CED9;}")
+        #self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+        #                                    "QProgressBar:chunk{background-color:#5DC4FE;}")#text-align:right;
+        self.ui.progressBar.setRange(0,100)
+        self.ui.progresslabel.setText(str(0) + '%')
+        self.ui.progressBar.reset()
+
+    def slot_progress_change(self, pkgname, percent, status):
+        if self.app.name == pkgname:
+            self.star.hide()
+            self.ui.progressBar.setVisible(True)
+            self.ui.progresslabel.setVisible(True)
+            self.ui.progressBar_icon.setVisible(True)
+            if status == AppActions.INSTALL:
+                self.ui.btn.setText("正在安装")
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#BBF9A3;}")
+            elif status == AppActions.UPGRADE:
+                self.ui.btn.setText("正在升级")
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                              "QProgressBar:chunk{background-color:#FDD99A;}")
+            elif status == AppActions.REMOVE:
+                self.ui.btn.setText("正在卸载")
+                self.ui.progressBar.setStyleSheet("QProgressBar{background-color:#F4F8FB;border:0px;border-radius:0px;color:#1E66A4;}"
+                                            "QProgressBar:chunk{background-color:#C5CED9;}")
+
+            self.ui.progressBar.setValue(percent)
+            if percent < float(0.0):
+                self.ui.progressBar.setValue(0)
+                self.ui.progresslabel.setText("失败")
+            else:
+                self.ui.progresslabel.setText(str('%.0f' % percent) + '%')
+
+    def slot_progress_finish(self,pkgname):
+        if self.app.name == pkgname:
+            self.ui.progresslabel.setVisible(False)
+            self.ui.progressBar_icon.setVisible(False)
+            self.ui.progressBar.setVisible(False)
+            self.ui.progressBar.reset()
+
+    def slot_progress_cancel(self,pkgname):
+        if self.app.name == pkgname:
+            self.ui.progresslabel.setVisible(False)
+            self.ui.progressBar_icon.setVisible(False)
+            self.ui.progressBar.setVisible(False)
+            self.ui.progressBar.reset()
+
     # kobe 1106
     def slot_change_btn_status(self, pkgname, status):
         if self.app.name == pkgname:
             self.ui.btn.setEnabled(False)
             if status == PkgStates.INSTALLING:
                 self.app.status = PkgStates.INSTALLING
-                self.ui.btn.setText("正在安装")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在安装")
+                else:
+                    self.ui.btn.setText("等待安装")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-install-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-install-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-install-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-install-border.png');}")
             elif status == PkgStates.REMOVING:
                 self.app.status = PkgStates.REMOVING
-                self.ui.btn.setText("正在卸载")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在卸载")
+                else:
+                    self.ui.btn.setText("等待卸载")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-un-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-un-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-un-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-un-border.png');}")
 
             elif status == PkgStates.UPGRADING:
                 self.app.status = PkgStates.UPGRADING
-                self.ui.btn.setText("正在升级")
+                if self.app.percent > 0:
+                    self.ui.btn.setText("正在升级")
+                else:
+                    self.ui.btn.setText("等待升级")
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
                 self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
 
@@ -309,8 +381,9 @@ class RcmdCard(QWidget):
                     self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-run-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-run-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-run-btn-3.png');}")
                     self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-run-border.png');}")
 
-    def slot_work_cancel(self, pkgname,action):
+    def slot_work_cancel(self, pkgname, action):
         if self.app.name == pkgname:
+
             if action == AppActions.INSTALL:
                 self.app.status = PkgStates.INSTALL
                 self.star.show()
@@ -339,4 +412,7 @@ class RcmdCard(QWidget):
                 self.ui.btn.setText("升级")
                 self.ui.btn.setEnabled(True)
                 self.ui.btn.setStyleSheet("QPushButton{color:white;border:0px;background-image:url('res/ncard-up-btn-1.png');}QPushButton:hover{border:0px;background-image:url('res/ncard-up-btn-2.png');}QPushButton:pressed{border:0px;background-image:url('res/ncard-up-btn-3.png');}")
-                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
\ No newline at end of file
+                self.ui.btnDetail.setStyleSheet("QPushButton{border:0px;background-image:url('res/ncard-up-border.png');}")
+
+            if self.app.percent < 0:
+                self.star.hide()
\ No newline at end of file
